-- Copyright (C) 2014  Fraser Tweedale
--
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
--
--      http://www.apache.org/licenses/LICENSE-2.0
--
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.

{-# OPTIONS_GHC -fno-warn-orphans #-}

module Crypto.JOSE.Types.Orphans where

import Prelude hiding (mapM)

import Data.Traversable

import Data.List.NonEmpty (NonEmpty(..), toList)
import qualified Data.Text as T
import qualified Data.Vector as V
import Network.URI (URI, parseURI)

import Data.Aeson


instance FromJSON a => FromJSON (NonEmpty a) where
  parseJSON = withArray "NonEmpty [a]" $ \v -> case V.toList v of
    [] -> fail "Non-empty list required"
    (x:xs) -> mapM parseJSON (x :| xs)

instance ToJSON a => ToJSON (NonEmpty a) where
  toJSON = Array . V.fromList . map toJSON . toList


instance FromJSON URI where
  parseJSON = withText "URI" $
    maybe (fail "not a URI") return . parseURI . T.unpack

instance ToJSON URI where
  toJSON = String . T.pack . show
